from Models import Model
import sys
import traceback

if __name__ == '__main__':

    directory=  "..\\data\\IMDB\\" # directory for datasets
    objDirectory = "..\\obj\\" # directory for objects (dumped models)
    obj = "imdb_rrn"

    """
    paths and hyper-parameters
    refer to README or Model.py for detailed descriptions
    """
    params = {
        'saveto': objDirectory+obj+"\\model_imdb.npz",
        'dataset': 'text', # {'text', 'mnist'}
        'corpus': directory+'imdb.pkl',
        'embedding': directory+'embedding.npz',
        'reload_model': objDirectory+obj+"\\model_imdb.npz",
        'reload_options': objDirectory+obj+"\\model_imdb.npz.pkl",
        'encoder': 'residual',
        'encoder2': None, # used for hybrid mode only
        'mean_pooling': False, # use last hidden state if set to False
        'hierarchical': False,
        'hybrid': False,
        'hier_len': None, # bottom level encoder length (for hierarchical-LSTM)
        'clip_c': 1.,
        'dropout_input': None,
        'dropout_output': None,
        'unit_depth': 2, # K deep transformation for RRN
        'skip_steps': -1, # skip length L for SC-LSTM
        'skip_steps2': -1,
        'truncate_grad': -1, # not applicable
        'n_words': 10000, # 10000 for IMDB and AGnews, 20000 for 20NG
        'dim_word': 256,
        'dim_proj': 512,
        'patience': 10,
        'maxlen': None, # use full length
        'batch_size': 64,
        'valid_batch_size': 64,
        'batch_len_threshold': None, # decrease batch size if length exceeded
        'dispFreq': 50,
        'validFreq': 200,
        'newDumpFreq': 10000,
        'start_iter': 0,
        'start_epoch': 0,
        'max_epochs': 10000,
        'lrate': 1., # not actually used in adadelta
        'decay_c': 0., # L2 regularization
        'noise_std': 0.,
        'optimizer': 'adadelta',
        'test_size': None,
        'monitor_grad': False, # print gradient norm to log file at each iteration if set True
        'logFile': objDirectory+obj+"\\log_"+obj+".txt"
    }

    try:
        model = Model(
                   dim_word=params['dim_word'],
                   dim_proj=params['dim_proj'],
                   patience=params['patience'],
                   max_epochs=params['max_epochs'],
                   dispFreq=params['dispFreq'],
                   decay_c=params['decay_c'],
                   clip_c=params['clip_c'],
                   lrate=params['lrate'],
                   n_words=params['n_words'],
                   encoder=params['encoder'],
                   encoder2=params['encoder2'],
                   optimizer=params['optimizer'],
                   hierarchical=params['hierarchical'],
                   hybrid=params['hybrid'],
                   mean_pooling=params['mean_pooling'],
                   hier_len=params['hier_len'],
                   unit_depth=params['unit_depth'],
                   skip_steps=params['skip_steps'],
                   skip_steps2=params['skip_steps2'],
                   truncate_grad=params['truncate_grad'],
                   saveto=params['saveto'],
                   validFreq=params['validFreq'],
                   newDumpFreq=params['newDumpFreq'],
                   maxlen=params['maxlen'],
                   batch_size=params['batch_size'],
                   valid_batch_size=params['valid_batch_size'],
                   batch_len_threshold=params['batch_len_threshold'],
                   corpus=params['corpus'],
                   dataset=params['dataset'],
                   embedding=params['embedding'],
                   noise_std=params['noise_std'],
                   dropout_output=params['dropout_output'],
                   dropout_input=params['dropout_input'],
                   reload_model=params['reload_model'],
                   reload_options=params['reload_options'],
                   test_size=params['test_size'],
                   start_iter=params['start_iter'],
                   start_epoch=params['start_epoch'],
                   monitor_grad=params['monitor_grad'],
                   logFile=params['logFile']
                   )
        train_acc, valid_acc, test_acc = model.train()

    except Exception:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, exc_traceback,
                                  limit=2, file=sys.stdout)